﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.ServiceModel.Syndication;
using ASPItalia.com.TicketTwo.Business;
using ASPItalia.com.TicketTwo.Contracts;
using ASPItalia.com.TicketTwo.Common.ObjectModel;
using System.ServiceModel.Web;

namespace ASPItalia.com.TicketTwo.Services
{

	// TODO: 05 Implementazione servizio WCF
    public class TicketTwoService : ITicketTwoService
    {

        #region ITicketTwoService Members

        /// <summary>
        /// Restituisce la lista delle ultime 5 esibizioni disponibili
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public SyndicationFeedFormatter GetLastExhibitions(string type)
        {
            FeedType ftype = (FeedType)Enum.Parse(typeof(FeedType), type);

            // Creo le informazioni principali del feed
            SyndicationFeed feed = new SyndicationFeed("TicketTwo", "Ultime esibizioni programmate", new Uri("http://www.tickettwo.it"));
            feed.Description = new TextSyndicationContent("Ultime esibizioni programmate");
            feed.Authors.Add(new SyndicationPerson("info@tickettwo.it"));

            // Popolo gli item
            List<SyndicationItem> items = new List<SyndicationItem>();

            feed.Items = (from e in new ExhibitionsManager().List()
                          orderby e.Date descending
                          select new SyndicationItem(String.Format("{0} - {1}", e.Tour.Name, e.Location),
                              e.Tour.Description,
                              new Uri(String.Format("http://www.tickettwo.it/Exhibitions/{0}", e.ID)))).Take(5);

            // Ritorno il formatter RSS 2.0 creato
            // sulla base del feed
            if (ftype == FeedType.RSS20)
                return new Rss20FeedFormatter(feed);
            else
                return new Atom10FeedFormatter(feed);
        }

        public List<Tour> ListTours()
        {
            return (from t in new ToursManager().List()
                    orderby t.ID descending
                    select t).ToList();
        }

        public List<Exhibition> ListExhibitions(int tourID)
        {
            List<Exhibition> le = (from e in new ExhibitionsManager().List()
                                  where e.TourID == tourID
                                  select e).ToList();

            return le;
        }

        public List<TicketRange> ListTicketRanges(int ExhibitionID)
        {
            return (from t in new TicketRangesManager().List()
                    where t.ExhibitionID == ExhibitionID
                    select t).ToList();
        }

        public void AddOrderItem(AddOrderItemRequest request)
        {
            OrdersManager om = new OrdersManager();
            var order = (from o in om.List()
                         where o.AnonymousID == request.SessionID
                         select o).FirstOrDefault();
            // Creo l'ordine se non c'è
            if (order == null)
            {
                order = new Order();
                order.AnonymousID = request.SessionID;
                order.DateTime = DateTime.Now;
				order.FullName = string.Empty;
                order.DeliveryAddress = String.Empty;
                order.DeliveryCity = String.Empty;
                order.DeliveryPostalCode = String.Empty;
                order.DeliveryState = String.Empty;
                om.Create(order);
            }

            // Aggiungo l'elemento all'ordine
            request.OrderItem.OrderID = order.ID;
            new OrderItemsManager().Create(request.OrderItem);
        }

        #endregion
    }
}
