﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ASPItalia.com.TicketTwo.Silverlight.Services;
using System.Windows.Markup;
using System.Windows.Browser;
using System.ComponentModel;

namespace ASPItalia.com.TicketTwo.Silverlight
{
    public partial class BuyTicket : UserControl
    {

        private TicketTwoManager ticketTwoManager;
        private const int colorOffset = 40;
        private static readonly DependencyProperty TicketRangeProperty = DependencyProperty.RegisterAttached("TicketRange", typeof(TicketRange), typeof(BuyTicket), null);

        public BuyTicket()
        {
            App app = (App)Application.Current;
            ticketTwoManager = app.TicketTwoManager;
            this.DataContext = ticketTwoManager;

            InitializeComponent();

            ((INotifyPropertyChanged)ticketTwoManager).PropertyChanged += new PropertyChangedEventHandler(BuyTicket_PropertyChanged);
			ticketTwoManager.AddOrderItemCompleted += new EventHandler<AsyncCompletedEventArgs>(ticketTwoManager_AddOrderItemCompleted);

        }

		public void Update()
		{
			//  Rimuovo i radio già esistenti
			while (exhibitionPanel.Children.Count > 1)
				exhibitionPanel.Children.RemoveAt(1);

			// Creo le RadioButton delle località
			foreach (Exhibition e in ticketTwoManager.Exhibitions)
			{
				RadioButton rb = new RadioButton();
				rb.GroupName = e.GetType().Name;
				rb.Content = e.City;
				exhibitionPanel.Children.Add(rb);
			}

			if (exhibitionPanel.Children.Count > 0)
				((RadioButton)exhibitionPanel.Children[1]).IsChecked = true;

			// TEMP: carico subito
			Exhibition ex = (from e in ticketTwoManager.Exhibitions
							 where e.Map != null
							 select e).FirstOrDefault();
			if (ex == null) return;

			// Seleziono la esibizione corrente
			ticketTwoManager.SelectedExhibition = ex;
		}

        void BuyTicket_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "TicketRanges")
                LoadExhibition(ticketTwoManager.SelectedExhibition);
        }

        private void LoadExhibition(Exhibition ex)
        {
			if (mapPanel.Children.Count > 1)
				mapPanel.Children.RemoveAt(1);
			
			if (ticketTwoManager.TicketRanges == null) return;

            // Carico la mappa XAML
            Panel map = (Panel)XamlReader.Load(ex.Map.ToString());
            Grid.SetColumn(map, 0);
            mapPanel.Children.Add(map);

            foreach (TicketRange tr in ticketTwoManager.TicketRanges)
            {
                UIElement range = (UIElement)map.FindName(tr.Ref);
                range.SetValue(TicketRangeProperty, tr);

                // Creo il tooltip per ogni area
                ToolTip t = new ToolTip();
                t.Content = tr.Name;
                ToolTipService.SetToolTip(range, t);
                ToolTipService.SetInitialShowDelay(range, 0);

                // Intercetto gli eventi del mouse
                range.MouseEnter += new MouseEventHandler(range_MouseEnter);
                range.MouseLeave += new MouseEventHandler(range_MouseLeave);
                range.MouseLeftButtonUp += new MouseButtonEventHandler(range_MouseLeftButtonUp);
            }
		
			mapIn.Begin();
		}

        void range_MouseLeave(object sender, MouseEventArgs e)
        {
            UIElement range = (UIElement)sender;
            TicketRange tr = (TicketRange)range.GetValue(TicketRangeProperty);

            Color c = ((SolidColorBrush)((Rectangle)range).Fill).Color;
            c.R -= colorOffset;
            c.G -= colorOffset;
            c.B -= colorOffset;
            ((Rectangle)range).Fill = new SolidColorBrush(c);
        }

        void range_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            UIElement range = (UIElement)sender;
            TicketRange tr = (TicketRange)range.GetValue(TicketRangeProperty);

            // Auto seleziono l'elemento nella ListBox
            ticketRanges.SelectedItem = tr;
        }

        void range_MouseEnter(object sender, MouseEventArgs e)
        {
            UIElement range = (UIElement)sender;
            TicketRange tr = (TicketRange)range.GetValue(TicketRangeProperty);

            // Semitrasparente
            Color c = ((SolidColorBrush)((Rectangle)range).Fill).Color;
            c.R += colorOffset;
            c.G += colorOffset;
            c.B += colorOffset;
            ((Rectangle)range).Fill = new SolidColorBrush(c);
        }

        private void AddToCard_Click(object sender, RoutedEventArgs e)
        {
			if (ticketTwoManager.SelectedExhibition == null
				|| ticketTwoManager.SelectedTicketRange == null) return;

			// TODO: 08 Aggiunta al carrello

			// Aggiungo l'elemento preparato nel carrello
            ticketTwoManager.AddToCard();
        }

		void ticketTwoManager_AddOrderItemCompleted(object sender, AsyncCompletedEventArgs e)
		{
			// Aggiornamento basket SL
			// TODO: 09 Refresh carrello
			HtmlPage.Window.Eval(String.Format("__doPostBack('{0}');", ((App)Application.Current).BasketUpdateID));

			// Messaggio di conferma
			HtmlPage.Window.Alert("Biglietti aggiunti nel carrello");
		}

        private void ticketRanges_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
                ticketTwoManager.SelectedTicketRange = (TicketRange)e.AddedItems[0];
        }

		private void GoBack_Click(object sender, RoutedEventArgs e)
		{
			((App)Application.Current).Navigation.NavigateToBack();
		}
    }
}