﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ASPItalia.com.TicketTwo.Silverlight.Services;
using System.ComponentModel;
using System.Threading;

namespace ASPItalia.com.TicketTwo.Silverlight
{
	// TODO: 06 Manager client Silverlight
    public class TicketTwoManager : INotifyPropertyChanged
    {

        private PropertyChangedEventHandler _notifyPropertyChanged;

        private TicketTwoServiceClient client;
        private Tour[] _tours;
        private Exhibition[] _Exhibitions;
        private Tour _selectedTours;
        private Exhibition _selectedExhibition;
        private TicketRange[] _ticketRanges;
        private TicketRange _selectedTicketRange;
        private int _quantityToBuy = 1;
        private string anonymousID;

        public TicketTwoManager(string anonymousID)
        {
            this.anonymousID = anonymousID;

            // Istanzio il client del servizio WCF
            client = new TicketTwoServiceClient();
            client.ListToursCompleted += new EventHandler<ListToursCompletedEventArgs>(client_ListToursCompleted);
            client.ListExhibitionsCompleted += new EventHandler<ListExhibitionsCompletedEventArgs>(client_ListExhibitionsCompleted);
            client.ListTicketRangesCompleted += new EventHandler<ListTicketRangesCompletedEventArgs>(client_ListTicketRangesCompleted);

            // Carico la lista dei tour
            // Valuto se sono in modalità design
            if (Application.Current.RootVisual != null && DesignerProperties.GetIsInDesignMode(Application.Current.RootVisual))
                this.Tours = new Tour[] { new Tour() };
            else
                client.ListToursAsync();
        }

		public event EventHandler<AsyncCompletedEventArgs> AddOrderItemCompleted
		{
			add { this.client.AddOrderItemCompleted += value; }
			remove { this.client.AddOrderItemCompleted -= value; }
		}
		

        void client_ListTicketRangesCompleted(object sender, ListTicketRangesCompletedEventArgs e)
        {
            this.TicketRanges = e.Result;
        }

        void client_ListExhibitionsCompleted(object sender, ListExhibitionsCompletedEventArgs e)
        {
            this.Exhibitions = e.Result;
        }

        void client_ListToursCompleted(object sender, ListToursCompletedEventArgs e)
        {
            this.Tours = e.Result;
        }

        /// <summary>
        /// Lista dei tour disponibili all'acquisto
        /// </summary>
        public Tour[] Tours
        {
            get { return _tours; }
            protected set
            {
                _tours = value;
                OnPropertyChanged("Tours");
            }
        }

        public Exhibition[] Exhibitions
        {
            get { return _Exhibitions; }
            protected set
            {
                _Exhibitions = value;
                OnPropertyChanged("Exhibitions");
            }
        }

        public TicketRange[] TicketRanges
        {
            get { return _ticketRanges; }
            protected set
            {
                _ticketRanges = value;
                OnPropertyChanged("TicketRanges");
            }
        }

        public int QuantityToBuy
        {
            get { return _quantityToBuy; }
            set
            {
                _quantityToBuy = value;
                OnPropertyChanged("QuantityToBuy");
            }
        }

        /// <summary>
        /// Indica il tour attualmente selezionato
        /// </summary>
        public Tour SelectedTour
        {
            get { return _selectedTours; }
            set
            {
                _selectedTours = value;
                OnPropertyChanged("SelectedTours");
                // Carico le esibizioni
                this.client.ListExhibitionsAsync(value.ID);
            }
        }

        /// <summary>
        /// Indica l'esibitione selezionato
        /// </summary>
        public Exhibition SelectedExhibition
        {
            get { return _selectedExhibition; }
            set
            {
                _selectedExhibition = value;
                OnPropertyChanged("SelectedExhibition");
                this.TicketRanges = null;
                this.client.ListTicketRangesAsync(value.ID);
            }
        }

        public TicketRange SelectedTicketRange
        {
            get { return _selectedTicketRange; }
            set
            {
                _selectedTicketRange = value;
                OnPropertyChanged("SelectedTicketRange");
            }
        }

        public void AddToCard()
        {
            // Aggiungo l'ordine al carrello tramite WebService
            OrderItem oi = new OrderItem();
            oi.Quantity = this.QuantityToBuy;
            oi.TicketRangeID = this.SelectedTicketRange.ID;
            oi.Amount = this.SelectedTicketRange.Price * oi.Quantity;

            client.AddOrderItemAsync(new AddOrderItemRequest()
            {
                SessionID = this.anonymousID,
                OrderItem = oi
            });
        }

        #region INotifyPropertyChanged Members

        protected virtual void OnPropertyChanged(string propertyName)
        {
            OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }

        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (_notifyPropertyChanged != null)
                _notifyPropertyChanged(this, e);
        }

        event PropertyChangedEventHandler INotifyPropertyChanged.PropertyChanged
        {
            add { _notifyPropertyChanged += value; }
            remove { _notifyPropertyChanged -= value; }
        }

        #endregion
    }
}
